function [phi modes] = ModalDecomp2(plant)
% Exact code:
% [phi modes] = ModalDecomp2(plant)
% ndof = size(plant.a,1)/2;
% invMK = -plant.a(ndof+1:2*ndof,1:ndof);
% [phi_temp modes] = eig(invMK);
% modes(1,:) = sqrt(diag(modes))/(2*pi); 
% modes(2:end,:) = [];
% [modes ind] = sort(modes);
% phi = phi_temp(:,ind);
%
% The function receives a state space system. Only the A matrix part of
% the state space is used here. There are two outputs, phi and modes. The
% first, phi, is the mode shape matrix. Each column corresponds to a mode
% frequency. The columns are sorted in order of ascending mode frequency.
% The rows correspond to the system's degrees of freedom. The second output
% is modes. This is a vector listing the mode frequencies in ascending
% order in units of Hz. Each value corresponds the respective column of phi.
%
% Example: The vertical (bounce) model of the monolithic quad.
% (this model is obtained with the model cut function, vertmod =
% model_cut(mbquad,'v');)
% [phi modes] = ModalDecomp2(vertmod)
% phi =
% 
%     0.2648   -0.8317   -0.5530   -0.0025
%     0.4732   -0.4730    0.8185    0.0526
%     0.5920    0.1924   -0.0894   -0.7196
%     0.5963    0.2178   -0.1278    0.6924
% 
% modes =
% 
%     0.5502    2.2162    3.5582    9.2697
% 
% The first column of phi is the mode shape of the 0.5502 Hz mode. Each row
% (unit length) represents the relative amplitude of motion for each stage. 
% The first row is the top mass, the second the UIM, the third the PUM, and
% the fourth, the test mass. The numbers here indicate that each stage is
% moving in phase with the others because they are all the same sign, and 
% each stage moves slightly more than the stage directly above. The second
% column represents the 2.2162 Hz mode. At this frequency the top two
% stages move in the opposite direction of the bottom two. Also, the top
% stages have a larger motion than the bottom two. At the frequency of the
% third mode, the motion is largely the top mass and UIM moving in opposite
% directions. The lower stages move a small amount the same way as the 
% top mass. At the fourth mode the top two stages have a negligible
% participation. Virtually all motion is the bottom two stages moving in
% opposite directions (bounce motion of the fibers). 
% 
% Notes:
% The function assumes the A matrix has the
% form of 
% A = [ zeros(ndof)    eye(ndof)
%         -M\K      -b*eye(ndof) ]
% Where ndof is the number of degrees of freedom, M is the mass matrix, K
% is the stiffness matrix and b is some quantity of velocity damping. All
% LIGO quad and triple suspension models have this form and can be used 
% with this function. The damping value is typically close to zero 
% (e.g. 0.01), if not exactly zero.
%
% Brett Shapiro
% Last updated: 22 November 2011


ndof = size(plant.a,1)/2;                % number of suspension degrees of freedom
invMK = -plant.a(ndof+1:2*ndof,1:ndof);  % negative of the lower left corner of the A state space matrix
[phi_temp modes] = eig(invMK);           % raw modes shapes and frequencies
modes(1,:) = sqrt(diag(modes))/(2*pi);   % mode frequencies are converted from units of radians to Hz
modes(2:end,:) = [];                        
[modes ind] = sort(modes);
phi = phi_temp(:,ind);