function plant = model_cut(mbquad,dof,varargin)
% plant = model_cut(mbquad,dof,varargin)
%
% Compatible with models generated by ssmake4pv2eMB4f and ssmake4pv2eMB5f.
% Compatibility with other model versions unknown.
%
% This function pulls apart the dynamics of the quad model into decoupled
% subsytems, so for example you can create a state space model of yaw
% alone with no information about the other degrees of freedom. Other 
% standard ones are long and z. It also has an option for the
% converse systems, xypr (x-y-pitch-roll), ypr (y-pitch-roll)
% 
% You can also consider the dynamics when individual masses are free.
% 'top' is when only the top mass is free, 'top_ui' is when the top and uim
% are free, 'top_ui_pum' (or 'top_ui_pen') is when the top, uim, and
% pum are free.
%
% The function assumes that the order of the degrees of freedom in the
% state space system follows long, trans, vert, yaw, pitch, roll. The order
% is repeated at each stage from the top to the bottom. The first 6 inputs
% also represent the motion of the seismic isloation table in the same order.
%
% List of DOF options:
% 'full' -> The full quad. No change is made to the model with this choice 
%           on its own. It is intended to be used with the seis_cut option 
%           to generate a triple pendulum with the quad's top mass locked. 
%           The top mass force inputs are changed to displacement/seismic 
%           inputs .
% 'l'    -> All 4 masses with only long DOFs free
% 'v'    -> All 4 masses with only z DOFs free (aka 'vert')
% 'y'    -> All 4 masses with only yaw DOFs free
% 'lp'   -> All 4 masses with only long and pitch DOFs free
% 'tr'   -> All 4 masses with only y and roll DOFs free
% 'tpr'  -> All 4 masses with y, pitch, and roll DOFs free
% 'ltpr' -> All 4 masses with x, y, pitch, and roll DOFs free (aka 'lptr')
% 'test' -> Only test mass free
% 'test_pum' -> Only test mass and PUM free. The seis_cut removes the PUM 
%               dofs and replaces them with seismic inputs for the single
%               pendulum of the test mass.
% 'test_pum_uim' -> Only top mass locked. The seis_cut removes the UIM 
%               dofs and replaces them with seismic inputs for the double
%               pendulum of the test mass and PUM.
% 'test_pum_uim_z' -> top mass locked, lower stages free in z only
% 'pum' -> only PUM free
% 'uim' -> only UIM free
% 'ui_z' -> UIM free in z only, all others locked
% 'top'  -> Only top mass free
% 'top_uim' -> Only top and UI masses free
% 'top_uim_pum' -> Top, UI, and PUM masses free (aka 'top_ui_pen')
% 'l_top_uim_pum' -> Top, UI, and PUM masses free in x(aka 'l_top_ui_pen')
% 'test_pum_lp' -> test mass and PUM free in long and pitch
% 'test_pum_y_roll' -> test mass and PUM free in y and roll
% 'test_pum_yaw' -> test mass and PUM free in yaw only
% 'test_pum_ui_yaw' -> test mass, PUM, and UIM free in yaw only
% 'test_lp' -> Only test mass free in long and pitch
% 'test_y' -> only test mass free in yaw
% 'l34_pitch4' -> Like 'test_lp', but the PUM is also free in long. The
%                 PUM long DOF is intended to be swapped for a seismic input
%                 with seis_cut.
% 'l234' -> test mass, PUM, and UIM free in x. The UIM long DOF is intended to
%           be swapped for a seismic input with seis_cut.
% 'l234_p34' -> PUM and test free in long and p, but UIM also free in long. 
%                   The UIM long DOF is intended to be swapped for a seismic 
%                   input with seis_cut.
% 'l1234_p234' -> UI, PUM, test, free in long and p, but top also free in 
%                     x. The top long DOF is intended to be swapped for a 
%                     seismic input with seis_cut.
%
% The optional input 'varargin' can only be used if it is set to
% 'seis_cut'. This input is only relevant for the last 4 options above
% ('x34_pitch4' to 'x1234_pitch234'), 'full', 'test_pum_uim', and 
% 'test_pum'. If used it removes and removes the DOFs at the highest
% modeled stage and swaps them for seismic inputs. Nominally the seismic 
% inputs only exist at the top mass, so this allows one to inject seismic
% inputs when no top mass is involved. For the default, without 'seis_cut',
% the code does not provide seismic inputs when the top mass is removed.
% As an example to run the code with this option, 'seis_cut' will convert 
% the force input only triple pendulum given by the 'test_pum_ui' choice to
% a double pendulum with both force and seismic inputs. This example is
% executed with the command
%   new_model = model_cut(original_model,'test_pum_ui','seis_cut')
%
% Brett Shapiro
% Last updated 22 November 2011
%
% Added cavity lock cases:
% 'l_top_uim_pum' -> only longitudinal dynamics of top, uim, and pum.
% Approximates a quad when a cavity locks the longitudinal test mass DOF.% 
% Brett Shapiro
% 25 September 2012

% Default seismic input state: 0 changes nothing.
seis_state = 0;

% bringing in the optional input for the seis_state parameter
if size(varargin,2) > 1
    error('Too many inputs')
elseif size(varargin,2) == 1
    seis_state = varargin{1}; % must equal 'seis_cut' to ensure this feature is not accidentally called
end

a = mbquad.a; b = mbquad.b; c = mbquad.c;

if strcmp(dof,'full') % does not change the model on its own; this option exists purely for the 'seis_cut' option, which in this case converts the top mass inputs to seismic inputs for the triple pendulum comprising the lower three stages 
    rem = [];
    remb = [];
    plant_dim = 24-6; % for the seis_cut option
elseif strcmp(dof,'x') || strcmp(dof,'l') % only long DOFs free at each mass
    rem = 2:6;
    rem = [rem 6+rem 12+rem 18+rem];
    remb = [2:6 6+rem];
elseif strcmp(dof,'vert') || strcmp(dof,'z') || strcmp(dof,'v') % only z DOFs free at each mass
    rem = [1 2 4 5 6];
    rem = [rem 6+rem 12+rem 18+rem];     
    remb = [1 2 4 5 6 6+rem];
elseif strcmp(dof,'yaw') || strcmp(dof,'y')% only yaw DOFs free at each mass
    rem = [1 2 3 5 6];
    rem = [rem 6+rem 12+rem 18+rem];
    remb = [1 2 3 5 6 6+rem];
elseif strcmp(dof,'xp') || strcmp(dof,'lp') % only long and pitch DOFs free at each mass
    rem = [2 3 4 6];
    rem = [rem 6+rem 12+rem 18+rem];    
    remb = [2 3 4 6 6+rem];
elseif strcmp(dof,'tr') % y and roll DOFs free at each mass
    rem = [1 3 4 5];
    rem = [rem 6+rem 12+rem 18+rem];
    remb = [1 3 4 5 6+rem];
elseif strcmp(dof,'tpr') || strcmp(dof,'ptr') % y, pitch, and roll DOFs free at each mass
    rem = [1 3 4];
    rem = [rem 6+rem 12+rem 18+rem];
    remb = [1 3 4 6+rem];
elseif strcmp(dof,'lptr') % x, y, pitch, and roll DOFs free at each mass
    rem = [3 4];
    rem = [rem 6+rem 12+rem 18+rem]; 
    remb = [3 4 6+rem];  
elseif strcmp(dof,'test') % only test mass free
    rem = 1:18;
    remb = [1:6,6+rem];
elseif strcmp(dof,'test_pum') % only test mass and PUM free
    rem = 1:12;
    remb = [1:6,6+rem];
    plant_dim = 12-6; % for the seis_cut option
elseif strcmp(dof,'test_pum_ui') || strcmp(dof,'test_pum_uim') || strcmp(dof,'test_pen_ui') || strcmp(dof,'test_pen_uim') % Top mass locked
    rem = 1:6;
    remb = [1:6,6+rem];
    plant_dim = 18-6; % for the seis_cut option
    dof = 'test_pum_uim'; % standardize this dof name for the rest of the code
elseif strcmp(dof,'test_pum_ui_z') || strcmp(dof,'test_pum_uim_z') || strcmp(dof,'test_pen_ui_z') || strcmp(dof,'test_pen_uim_z') % Top mass locked, lower stages free in z only
    rem = [1:8 10:14 16:20 22:24];
    remb = [1:6,6+rem];
elseif strcmp(dof,'pum') % only PUM free
    rem = [1:12 19:24];
    remb = [1:6,6+rem];
elseif strcmp(dof,'ui') || strcmp(dof,'uim') % only UIM free
    rem = [1:6 13:24];
    remb = [1:6,6+rem];
elseif strcmp(dof,'ui_z') || strcmp(dof,'uim_z') % only UIM free in z
    rem = [1:8 10:24];
    remb = [1:6,6+rem];
elseif strcmp(dof,'top') % only top mass free
    rem = 7:24;
    remb = 6+rem;    
elseif strcmp(dof,'top_ui') || strcmp(dof,'top_uim')% only top and UI free
    rem = 13:24;
    remb = 6+rem;  
elseif strcmp(dof,'top_ui_pen') || strcmp(dof,'top_ui_pum') || strcmp(dof,'top_uim_pen') || strcmp(dof,'top_uim_pum')% top, ui, and penultimate are free
    rem = 19:24;
    remb = 6+rem;
elseif strcmp(dof,'l_top_ui_pen') || strcmp(dof,'l_top_ui_pum') || strcmp(dof,'l_top_uim_pen') || strcmp(dof,'l_top_uim_pum')% top, ui, and penultimate are free in longitudinal
    rem = [2 3 4 5 6 8 9 10 11 12 14:24];
    remb = [2:6 6+rem];
elseif strcmp(dof,'test_pum_long_pitch') || strcmp(dof,'test_pum_lp') || strcmp(dof,'test_pum_xp') || strcmp(dof,'test_pum_x_pitch') % test mass and PUM free in long and pitch
    rem = [1:12,14:16,18,20:22,24];
    remb = [1:6,6+rem];
elseif strcmp(dof,'test_pum_trans_roll') || strcmp(dof,'test_pum_tr')% test mass and PUM free in y and roll
    rem = [1:13,15:17,19,21:23];
    remb = [1:6,6+rem];
elseif strcmp(dof,'test_pum_yaw') || strcmp(dof,'test_pum_y') % test mass and PUM free in yaw only
    rem = [1:15,17:21,23,24];
    remb = [1:6,6+rem];
elseif strcmp(dof,'test_pum_ui_yaw') || strcmp(dof,'test_pum_uim_yaw') || strcmp(dof,'test_pen_ui_yaw') ||  strcmp(dof,'test_pen_uim_yaw') || strcmp(dof,'test_pum_uim_y')% test mass, PUM, and UIM free in yaw only
    rem = [1:9,11:15,17:21,23,24];
    remb = [1:6,6+rem];
elseif strcmp(dof,'test_long_pitch') || strcmp(dof,'test_lp')% only test mass free in long and pitch
    rem = [1:18,20:22,24];
    remb = [1:6,6+rem]; 
elseif strcmp(dof,'test_yaw') || strcmp(dof,'test_y') % only test mass free in yaw
    rem = [1:21,23,24];
    remb = [1:6,6+rem];
elseif strcmp(dof,'l34_p4') % only test mass free in long and pitch, and long of the PUM mass (PUM intended as an effective long seismic input to test mass)
    rem = [1:12,14:18,20:22,24];
    remb = [1:6,6+rem];
    plant_dim = 2;
elseif strcmp(dof,'l234') % test mass, PUM, and UIM free in long (x of the UIM intended as an effective long seismic input to PUM).
    rem = [1:6,8:12,14:18,20:24];
    remb = [1:6,6+rem];
    plant_dim = 2;
elseif strcmp(dof,'l234_p34') % test mass and PUM free in long and pitch, and long of the UI mass (UI intended as an effective long seismic input to PUM)
    rem = [1:6,8:12,14:16,18,20:22,24];
    remb = [1:6,6+rem];
    plant_dim = 4;
elseif strcmp(dof,'l1234_p234') % test mass, PUM, and UI free in long and pitch, and long of the top mass (top mass intended as an effective long seismic input to UI)
    rem = [2 3 4 6];
    rem = [2:6 6+rem 12+rem 18+rem];
    remb = [1:6,6+rem];
    plant_dim = 6;
else
    error('Invalid DOF')
end

% rebuilding the state space matrices
rema = [rem 24+rem];
a(rema,:) = [];
a(:,rema) = [];
b(rema,:) = [];
b(:,remb) = [];
c(rem,:) = [];
c(:,rema) = [];
d = zeros(size(c,1),size(b,2)); 

% creating a new seismic input if asked for
if ischar(seis_state) && strcmp(seis_state,'seis_cut')
    if strcmp(dof,'l34_p4') || strcmp(dof,'l234') || strcmp(dof,'l234_p34') || strcmp(dof,'l1234_p234')
        b0_coef = a(plant_dim+3:end,1);
        a([1 plant_dim+2],:) = []; a(:,[1 plant_dim+2]) = [];
        b([1 plant_dim+2],:) = [];
        b(:,1) = [zeros(plant_dim,1);b0_coef];
        c(1,:) = []; c(:,[1 plant_dim+2]) = [];
        d = zeros(size(c,1),size(b,2));  
    elseif strcmp(dof,'full') || strcmp(dof,'test_pum_uim') || strcmp(dof,'test_pum')
        if strcmp(dof,'full')
            b(:,1:6) = []; % remove seismic table displacement inputs from full suspension
        end
        b0_coef = a(plant_dim+13:end,1:6);
        a([1:6 plant_dim+7:plant_dim+12],:) = []; a(:,[1:6 plant_dim+7:plant_dim+12]) = [];
        b([1:6 plant_dim+7:plant_dim+12],:) = [];
        b(:,1:6) = [zeros(plant_dim,6);b0_coef];
        c(1:6,:) = []; c(:,[1:6 plant_dim+7:plant_dim+12]) = [];
        d = zeros(size(c,1),size(b,2));  
    else
        error('This DOF field has no ''seis_cut'' option.')
    end
end

% % % Adding a little bit of damping and compiling the state space system
% % ndof = size(c,1);
% % a(ndof+1:2*ndof,ndof+1:2*ndof) = -0.001*eye(ndof);
plant = ss(a,b,c,d);