function [in,out] = define_quadModel_insandouts(quadState)
% [in,out] = define_quadModel_insandouts(quadState)
% 
% Defines indices for inputs (in) and outputs (out) for either an full open
% loop mbquad (24 x 30) or a full closed loop quad (30 x 36).
%
% INPUTS:
%       quadState: [1xn] string     defines in what state the model is.
%                                   Known quadStates:
%                                       'openloop'
%                                       'damped'
%
% OUTPUT:
%       in:        [1x1] struct
%       out:       [1x1] struct
%   Both in and out are structures of the following format:
%         in.(isoStage).(inputType).(dof)
%   where 
%         isoStage: isolation stage to which ins and outs are connected 
%                 = {'gnd';...   % At sus. point
%                    'top';...   % M0 or R0, top mass
%                    'uim';...   % "L1", upper intermediate mass
%                    'pum';...   % "L2", pen-ultimate mass
%                    'tst'};     % "L3", test mass
%         inputType: loop location (and there units)
%                 = {'disp';...  % displacement units ([m] or [rad]) 
%                    'drive';... % force / torque units ([N] or [N.m])
%                    'sensNoise'};  % in has units of displacement and out has
%                                % units of force (used for quadState =
%                                % 'damped' only) 
%         dof: coordinates are with respect to the optical axis / beam line
%                 = {'L';...     % longitudinal
%                    'T';...     % transverse
%                    'V';...     % vertical
%                    'R';...     % roll
%                    'P';...     % pitch
%                    'Y'};       % yaw
%
%
% NOTE: The order of angle indices looks screwy because Mark's Mathematica
% models are ordered Yaw, Pitch, Roll). Where a distinction is required
% (i.e. in the closed loop case) I clearly call out "standard" (R, P, Y)
% vs. "flipped" (Y, P, R) order in the adjacent comments.

switch quadState
    case 'openloop' % All Flipped Order (L T V Y P R ins and outs)
        % Ground noise input
        in.gnd.disp.L         = 1;    %[m]
        in.gnd.disp.T         = 2;    %[m]
        in.gnd.disp.V         = 3;    %[m]
        in.gnd.disp.R         = 6;    %[rad]
        in.gnd.disp.P         = 5;    %[rad]
        in.gnd.disp.Y         = 4;    %[rad]
        
        % Control force / torque input
        in.top.drive.L        = 7;    %[N]
        in.top.drive.T        = 8;    %[N]
        in.top.drive.V        = 9;    %[N]
        in.top.drive.R        = 12;   %[N m]
        in.top.drive.P        = 11;   %[N m]
        in.top.drive.Y        = 10;   %[N m]
        
        in.uim.drive.L        = 13;   %[N]
        in.uim.drive.T        = 14;   %[N]
        in.uim.drive.V        = 15;   %[N]
        in.uim.drive.R        = 18;   %[N m]
        in.uim.drive.P        = 17;   %[N m]
        in.uim.drive.Y        = 16;   %[N m]
        
        in.pum.drive.L        = 19;   %[N]
        in.pum.drive.T        = 20;   %[N]
        in.pum.drive.V        = 21;   %[N]
        in.pum.drive.R        = 24;   %[N m]
        in.pum.drive.P        = 23;   %[N m]
        in.pum.drive.Y        = 22;   %[N m]
        
        in.tst.drive.L        = 25;   %[N]
        in.tst.drive.T        = 26;   %[N]
        in.tst.drive.V        = 27;   %[N]
        in.tst.drive.R        = 30;   %[N m]
        in.tst.drive.P        = 29;   %[N m]
        in.tst.drive.Y        = 28;   %[N m]
        
        % Mass displacement output
        out.top.disp.L        = 1;    %[m]
        out.top.disp.T        = 2;    %[m]
        out.top.disp.V        = 3;    %[m]
        out.top.disp.R        = 6;    %[rad]
        out.top.disp.P        = 5;    %[rad]
        out.top.disp.Y        = 4;    %[rad]
        
        out.uim.disp.L        = 7;    %[m]
        out.uim.disp.T        = 8;    %[m]
        out.uim.disp.V        = 9;    %[m]
        out.uim.disp.R        = 12;   %[rad]
        out.uim.disp.P        = 11;   %[rad]
        out.uim.disp.Y        = 10;   %[rad]
        
        out.pum.disp.L        = 13;   %[m]
        out.pum.disp.T        = 14;   %[m]
        out.pum.disp.V        = 15;   %[m]
        out.pum.disp.R        = 18;   %[rad]
        out.pum.disp.P        = 17;   %[rad]
        out.pum.disp.Y        = 16;   %[rad]
        
        out.tst.disp.L        = 19;   %[m]
        out.tst.disp.T        = 20;   %[m]
        out.tst.disp.V        = 21;   %[m]
        out.tst.disp.R        = 24;   %[rad]
        out.tst.disp.P        = 23;   %[rad]
        out.tst.disp.Y        = 22;   %[rad]
    case 'damped' % Mixed order (each are called out explicitly, and are intentional)  
        % Sensor noise input (through damping filters, in displacement
        % units) 
        in.top.sensNoise.L       = 1;   % [m]
        in.top.sensNoise.T       = 2;   % [m]
        in.top.sensNoise.V       = 3;   % [m]
        in.top.sensNoise.R       = 4;   % [rad] | Standard Order
        in.top.sensNoise.P       = 5;   % [rad] |
        in.top.sensNoise.Y       = 6;   % [rad] V
        
        % Ground noise input
        in.gnd.disp.L         = 7;   % [m]
        in.gnd.disp.T         = 8;   % [m]
        in.gnd.disp.V         = 9;   % [m]
        in.gnd.disp.R         = 12;  % [rad] | Flipped order
        in.gnd.disp.P         = 11;  % [rad] |
        in.gnd.disp.Y         = 10;  % [rad] V
        
        % Control force input
        in.top.drive.L        = 13;  % [N]
        in.top.drive.T        = 14;  % [N]
        in.top.drive.V        = 15;  % [N]
        in.top.drive.R        = 18;  % [N.m] | Flipped order
        in.top.drive.P        = 17;  % [N.m] |
        in.top.drive.Y        = 16;  % [N.m] V
        
        in.uim.drive.L        = 19;  % [N]
        in.uim.drive.T        = 20;  % [N]
        in.uim.drive.V        = 21;  % [N]
        in.uim.drive.R        = 24;  % [N.m] | Flipped order
        in.uim.drive.P        = 23;  % [N.m] |
        in.uim.drive.Y        = 22;  % [N.m] V
        
        in.pum.drive.L        = 25;  % [N]
        in.pum.drive.T        = 26;  % [N]
        in.pum.drive.V        = 27;  % [N]
        in.pum.drive.R        = 30;  % [N.m] | Flipped order
        in.pum.drive.P        = 29;  % [N.m] |
        in.pum.drive.Y        = 28;  % [N.m] V
        
        in.tst.drive.L        = 31;  % [N]
        in.tst.drive.T        = 32;  % [N]
        in.tst.drive.V        = 33;  % [N]
        in.tst.drive.R        = 36;  % [N.m] | Flipped order
        in.tst.drive.P        = 35;  % [N.m] |
        in.tst.drive.Y        = 34;  % [N.m] V
        
        % Filter output (in force units)
        out.top.sensNoise.L = 1;  % [N]
        out.top.sensNoise.T = 2;  % [N]
        out.top.sensNoise.V = 3;  % [N]
        out.top.sensNoise.R = 4;  % [N.m] | Standard order
        out.top.sensNoise.P = 5;  % [N.m] |
        out.top.sensNoise.Y = 6;  % [N.m] V
        
        % Mass displacement output
        out.top.disp.L = 7;   % [m]
        out.top.disp.T = 8;   % [m]
        out.top.disp.V = 9;   % [m]
        out.top.disp.R = 12;  % [rad] | Flipped order
        out.top.disp.P = 11;  % [rad] |
        out.top.disp.Y = 10;  % [rad] V
        
        out.uim.disp.L = 13;  % [m]
        out.uim.disp.T = 14;  % [m]
        out.uim.disp.V = 15;  % [m]
        out.uim.disp.R = 18;  % [rad] | Flipped order
        out.uim.disp.P = 17;  % [rad] |
        out.uim.disp.Y = 16;  % [rad] V
        
        out.pum.disp.L = 19;  % [m]
        out.pum.disp.T = 20;  % [m]
        out.pum.disp.V = 21;  % [m]
        out.pum.disp.R = 24;  % [rad] | Flipped order
        out.pum.disp.P = 23;  % [rad] |
        out.pum.disp.Y = 22;  % [rad] V
        
        out.tst.disp.L = 25;  % [m]
        out.tst.disp.T = 26;  % [m]
        out.tst.disp.V = 27;  % [m]
        out.tst.disp.R = 30;  % [rad] | Flipped order
        out.tst.disp.P = 29;  % [rad] |
        out.tst.disp.Y = 28;  % [rad] V
    otherwise
        warning('Unrecognized state, defaulting to open loop.') %#ok<*WNTAG>
end